@@ -31,9 +31,9 @@ use Sys::Info::Constants qw( :device_cpu );
 
 use Exporter 'import';
 our @EXPORT = qw(system_identity suite_run calc_scalability);
-our $datadir = dist_dir("Benchmark-DKbench");
+my $datadir = dist_dir("Benchmark-DKbench");
 
-our $VERSION = '2.0';
+our $VERSION = '2.0_001';
 
 =head1 NAME
 
@@ -57,29 +57,43 @@ Benchmark::DKbench - Perl Benchmark
 =head1 DESCRIPTION
 
 A Perl benchmark suite for general compute, created to evaluate the comparative
-performance of systems when running computationally intensive Perl (both pure Perl
-and XS) workloads. Runs single and multi-threaded and can be fully customized to
-the benchmarks that better suit your own scenario.
+performance of systems when running the types of Perl workloads that typically
+cause CPU bottlenecks. Can run single or multi-threaded.
 
 The only non-CPAN libraries required to install/run the suite are libdb, libjpeg
-(with headers). There are a few prerequisite CPAN libraries, plus an optional one
-(L<BioPerl>, requires libxml on system) which enables extra benchmarks.
+(with headers). Of the CPAN libraries used, BioPerl is optional (requires libxml).
 
 =head1 SCRIPTS
 
 You should only ever need the main script C<dkbench> for the benchmark suite, and,
-perhaps C<setup_dkbench> can help with setup or standardizing/normalizing your
-benchmarking environment.
+perhaps C<setup_dkbench> can help with setup or normalizing your benchmarking
+environment.
 
 =head2 C<dkbench>
 
-The main script that runs the DKbench benchmark suite. If L<BioPerl> is installed,
-you may want to start your first run with C<dkbench --setup>. Otherwise, there are
-many options to control number of threads, iterations, which benchmarks to run etc:
+The main script that runs the DKbench benchmark suite. The default run (no options)
+will run all the benchmarks both single-threaded and multi-threaded (using all
+detected CPU cores/hyperthreads) and show you scores and multi vs single threaded
+scalability.
 
- dkbench [options]
+The scores are such that a reference CPU (Intel Xeon Platinum 8481C - Sapphire Rapids)
+would get 1000 on a single core benchmark run using the default software configuration
+(Linux/Perl 5.36.0/ref CPAN modules).
+
+The multi-thread scalability should approach 100% if each thread runs on a full core
+(i.e. no SMT), and the core can maintain the clock speed it had on the single-thread
+runs. Note that the overall scalability is an average of the benchmarks that drops
+non-scaling outliers (over 2*stdev less than the mean).
+
+The suite will report a Pass/Fail per benchmark. A failure may be caused if you have
+different CPAN module version installed - this is normal, and you will be warned.
+
+The suite uses L<MCE::Loop> to run on the desired number of parallel threads,
+although by default it will run on a single thread.
+
+There are many options to control number of threads, iterations, which benchmarks
+to run etc.
 
- Options:
  --threads <i>, -j <i> : Number of benchmark threads (default is 1).
  --multi,       -m     : Multi-threaded using all your CPU cores/threads.
  --iter <i>,    -i <i> : Number of suite iterations (with min/max/avg at the end).
@@ -96,33 +110,8 @@ many options to control number of threads, iterations, which benchmarks to run e
  --setup               : Download the Genbank data to enable the BioPerl tests.
  --datapath <path>     : Override the path where the expected benchmark data is found.
 
-The default run (no options) will run all the benchmarks both single-threaded and
-multi-threaded (using all detected CPU cores/hyperthreads) and show you scores and
-multi vs single threaded scalability.
-
-The scores are such that a reference CPU (Intel Xeon Platinum 8481C - Sapphire Rapids)
-would get 1000 on a single core benchmark run using the default software configuration
-(Linux/Perl 5.36.0/ref CPAN modules).
-
-The multi-thread scalability should approach 100% if each thread runs on a full core
-(i.e. no SMT), and the core can maintain the clock speed it had on the single-thread
-runs. Note that the overall scalability is an average of the benchmarks that drops
-non-scaling outliers (over 2*stdev less than the mean).
-
-The suite will report a Pass/Fail per benchmark. A failure may be caused if you have
-different CPAN module version installed - this is normal, and you will be warned.
-
-The suite uses L<MCE::Loop> to run on the desired number of parallel threads, although
-there is an option to disable it, which forces a single-thread run.
-
 =head2 C<setup_dkbench>
 
-Simple installer to check/get the reference versions of CPAN modules and download
-the Genbank data file required for the BioPerl benchmarks of the DKbench suite.
-
-It assumes that you have some software already installed (e.g. C<cpanm>), try
-C<setup_dkbench --help> for more details on that.
-
  setup_dkbench [--force --sudo --test --data=s --help]
 
  Options:
@@ -132,11 +121,16 @@ C<setup_dkbench --help> for more details on that.
  --data=s : Data dir path to copy files from. Should not need if you installed DKbench.
  --help   : Print this help text and exit.
 
-Running it without any options will fetch the data for the BioPerl tests (similar to
-C<dkbench --setup>) and use C<cpanm> to install any missing libraries.
+Simple installer to check/get the reference versions of CPAN modules and download
+the Genbank data file required for the BioPerl benchmarks of the DKbench suite.
 
-Using it with C<--force> will install the reference CPAN module versions, including
-BioPerl which is not a requirement for DKbench, but enables the BioPerl benchmarks.
+It assumes that you have some software already installed (e.g. C<cpanm>), try
+C<setup_dkbench --help> for more details.
+
+Running it without any options is the same as C<dkbench --setup>: it will simply
+fetch the data for the BioPerl tests. Using it with C<--force> will install the
+reference CPAN module versions, including BioPerl which is not a requirement for
+DKbench, but enables the BioPerl benchmarks.
 
 The reference CPAN versions are suggested if you want a fair comparison between
 systems and also for the benchmark Pass/Fail results to be reliable.
@@ -277,26 +271,48 @@ the same terms as the Perl 5 programming language system itself.
 =cut
 
 sub benchmark_list {
-    return {               # idx : 0 = result, 1 = ref time, 2 = func, 3 = quick test, 4 = normal test
-        'Astro'            => ['e71c7ae08f16fe26aea7cfdb72785873', 5.674, \&bench_astro, 20000, 80000],
-        'BioPerl Codons'   => ['97c443c099886ca60e99f7ab9df689b5', 8.752, \&bench_bioperl_codons, 3, 5, 1],
-        'BioPerl Monomers' => ['d29ed0a5c205c803c112be1338d1f060', 5.241, \&bench_bioperl_mono, 6, 20],
-        'Crypt::JWT'       => ['d41d8cd98f00b204e9800998ecf8427e', 6.451, \&bench_jwt, 250, 900],
-        'CSS::Inliner'     => ['82c1b6de9ca0500a48f8a8df0998df3c', 4.603, \&bench_css, 2, 5],
-        'DateTime'         => ['b08d2eeb994083b7422f6c9d86fed2c6', 6.198, \&bench_datetime, 5000, 15000],
-        'Digest'           => ['4b69f6cf0f53cbf6c3444f2f767dd21d', 4.513, \&bench_digest, 50, 250],
-        'Encode'           => ['PASS 1025',                        5.725, \&bench_encode, 40, 120],
-        'HTML::FormatText' => ['8c2589f0a5276252805e11301fc2ab56', 4.756, \&bench_formattext, 4, 10],
-        'Imager'           => ['8829cb3703e884054eb025496f336c63', 6.783, \&bench_imager, 4, 16],
-        'JSON::XS'         => ['PASS',                             5.388, \&bench_json, 600, 2200],
-        'Math::DCT'        => ['766e3bfd7a2276f452bb3d1bd21939bc', 7.147, \&bench_dct, 25000, 100_000],
-        'Math::MatrixReal' => ['4606231b1309fb21ae1223fa0043fd76', 4.293, \&bench_matrixreal, 200, 650],
-        'Moose'            => ['d1cb92c513f6378506dfa11f694cffac', 4.968, \&bench_moose, 10_000, 30_000],
-        'Moose prove'      => ['PASS',                             7.974, \&bench_moose_prv, 0.5, 1],
-        'Primes'           => ['4266f70a7a9efb3484cf5d98eba32244', 3.680, \&bench_primes_m, 2, 5],
-        'Regex/Subst'      => ['30ce365b25f3d597578b3bdb14aa3f57', 4.652, \&bench_regex_asc, 8, 24],
-        'Regex/Subst utf8' => ['857eb4e63a4d174ca4a16fe678f7626f', 5.703, \&bench_regex_utf8, 3, 10],
-        'Time::Piece'      => ['5d393b000a42a626bad7208f8be4121e', 5.907, \&bench_timepiece, 75_000, 275_000],
+    my $quick = shift;
+    return {
+        'Astro'            => sub {bench_astro($quick ? 20000 : 80000) },
+        'BioPerl Codons'   => sub {bench_bioperl_codons(1, $quick ? 3 : 5) },
+        'BioPerl Monomers' => sub {bench_bioperl_mono($quick ? 6 : 20) },
+        'Crypt::JWT'       => sub {bench_jwt($quick ? 250 : 900) },
+        'CSS::Inliner'     => sub {bench_css($quick ? 2 : 5) },
+        'DateTime'         => sub {bench_datetime($quick ? 5000 : 15000) },
+        'Digest'           => sub {bench_digest($quick ? 50 : 250) },
+        'Encode'           => sub {bench_encode($quick ? 40 : 120) },
+        'HTML::FormatText' => sub {bench_formattext($quick ? 4 : 10) },
+        'Imager'           => sub {bench_imager($quick ? 4 : 16) },
+        'JSON::XS'         => sub {bench_json($quick ? 600 : 2200) },
+        'Math::DCT'        => sub {bench_dct($quick ? 25000 : 100_000) },
+        'Math::MatrixReal' => sub {bench_matrixreal($quick ? 200 : 650) },
+        'Moose'            => sub {bench_moose($quick ? 10_000 : 30_000) },
+        'Moose prove'      => sub {bench_moose_tests($quick ? 0.5 : 1) },
+        'Primes'           => sub {bench_primes($quick ? 2 : 5, 7_500_000) },
+        'Regex/Subst'      => sub {bench_regex_repl($quick ? 8 : 24) },
+        'Regex/Subst utf8' => sub {bench_regex_repl($quick ? 3 : 10, 'utf8') },
+        'Time::Piece'      => sub {bench_timepiece($quick ? 75_000 : 275_000) },
+    },
+    {
+        'Astro'            => ['e71c7ae08f16fe26aea7cfdb72785873', 5.674],
+        'BioPerl Codons'   => ['97c443c099886ca60e99f7ab9df689b5', 8.752],
+        'BioPerl Monomers' => ['d29ed0a5c205c803c112be1338d1f060', 5.241],
+        'CSS::Inliner'     => ['82c1b6de9ca0500a48f8a8df0998df3c', 4.603],
+        'Crypt::JWT'       => ['d41d8cd98f00b204e9800998ecf8427e', 6.451],
+        'DateTime'         => ['b08d2eeb994083b7422f6c9d86fed2c6', 6.198],
+        'Digest'           => ['4b69f6cf0f53cbf6c3444f2f767dd21d', 4.513],
+        'Encode'           => ['PASS 1025',                        5.725],
+        'HTML::FormatText' => ['8c2589f0a5276252805e11301fc2ab56', 4.756],
+        'Imager'           => ['8829cb3703e884054eb025496f336c63', 6.783],
+        'JSON::XS'         => ['PASS',                             5.388],
+        'Math::DCT'        => ['766e3bfd7a2276f452bb3d1bd21939bc', 7.147],
+        'Math::MatrixReal' => ['4606231b1309fb21ae1223fa0043fd76', 4.293],
+        'Moose'            => ['d1cb92c513f6378506dfa11f694cffac', 4.968],
+        'Moose prove'      => ['PASS',                             7.974],
+        'Primes'           => ['4266f70a7a9efb3484cf5d98eba32244', 3.680],
+        'Regex/Subst'      => ['30ce365b25f3d597578b3bdb14aa3f57', 4.652],
+        'Regex/Subst utf8' => ['857eb4e63a4d174ca4a16fe678f7626f', 5.703],
+        'Time::Piece'      => ['5d393b000a42a626bad7208f8be4121e', 5.907],
     };
 }
 
@@ -353,7 +369,7 @@ sub suite_run {
 
 sub calc_scalability {
     my ($opt, $stats1, $stats2) = @_;
-    my $benchmarks = benchmark_list();
+    my ($benchmarks, $bench_results) = benchmark_list($opt->{quick});
     my $threads = $stats2->{threads}/$stats1->{threads};
     my $display = $opt->{time} ? 'times' : 'scores';
     $opt->{f} = $opt->{time} ? '%.3f' : '%5.0f';
@@ -366,7 +382,7 @@ sub calc_scalability {
         my @res2 = min_max_avg($stats2->{$bench}->{times});
         push @perf, $res1[2]/$res2[2]*$threads if $res2[2];
         push @scal, $res1[2]/$res2[2]*100 if $res2[2];
-        print pad_to("$bench:",24).pad_to(sprintf("%.2f",$perf[-1]),24).pad_to(sprintf("%2.0f",$scal[-1]),24)."\n";
+        print pad_to("$bench:",24).pad_to(sprintf("%.2f",$perf[-1]),24).pad_to(sprintf("%2d",$scal[-1]),24)."\n";
     }
     print (("-"x34)."\n");
     my ($avg1) = min_max_avg($stats1->{total}->{$display});
@@ -379,16 +395,14 @@ sub calc_scalability {
     @perf = min_max_avg(\@newperf);
     @scal = min_max_avg(\@newscal);
     print pad_to("Multi/Single perf:").sprintf("%.2fx\t(%.2f - %.2f)", $perf[2], $perf[0], $perf[1])."\n";
-    print pad_to("Multi scalability:").sprintf("%2.1f%% \t(%.0f%% - %.0f%%)", $scal[2], $scal[0], $scal[1])."\n";
+    print pad_to("Multi scalability:").sprintf("%2d%% \t(%d%% - %d%%)", $scal[2], $scal[0], $scal[1])."\n";
 }
 
 sub run_iteration {
     my ($opt, $stats) = @_;
-    my $benchmarks = benchmark_list();
+    my ($benchmarks, $bench_results) = benchmark_list($opt->{quick});
     my $title = $opt->{time} ? 'Time (sec)' : 'Score';
-    print pad_to("Benchmark").pad_to($title);
-    print "Pass/Fail" unless $opt->{time};
-    print "\n";
+    print pad_to("Benchmark").pad_to($title)."Pass/Fail\n";
     my ($total_score, $total_time, $i) = (0, 0, 0);
     foreach my $bench (sort keys %$benchmarks) {
         next if $opt->{skip_bio} && $bench =~ /Monomers/;
@@ -401,8 +415,8 @@ sub run_iteration {
             require Bio::SeqIO;
             require Bio::Tools::SeqStats;
          }
-        my ($time, $res) = mce_bench_run($opt, $benchmarks->{$bench});
-        my $score = int(1000*$opt->{threads}*$benchmarks->{$bench}->[1]/($time || 1)+0.5);
+        my ($time, $res) = mce_bench_run($opt, $benchmarks->{$bench}, $opt->{quick} ? undef : $bench_results->{$bench});
+        my $score = int(1000*$opt->{threads}*$bench_results->{$bench}->[1]/($time || 1)+0.5);
         $total_score += $score;
         $total_time += $time;
         $i++;
@@ -410,9 +424,7 @@ sub run_iteration {
         push @{$stats->{$bench}->{scores}}, $score;
         my $d = $stats->{$bench}->{$opt->{time} ? 'times' : 'scores'}->[-1];
         $stats->{$bench}->{fail}++ if $res ne 'Pass';
-        print pad_to("$bench:").pad_to(sprintf($opt->{f}, $d));
-        print "$res" unless $opt->{time};
-        print "\n";
+        print pad_to("$bench:").pad_to(sprintf($opt->{f}, $d))."$res\n";
         sleep $opt->{sleep} if $opt->{sleep};
     }
     die "No tests to run\n" unless $i;
@@ -423,15 +435,14 @@ sub run_iteration {
 }
 
 sub mce_bench_run {
-    my $opt       = shift;
-    my $benchmark = shift;
-    $benchmark->[3] = $benchmark->[4] unless $opt->{quick};
-    return bench_run($benchmark) if $opt->{no_mce};
+    my $opt = shift;
+    my @params = @_;
+    return bench_run(@params) if $opt->{no_mce};
 
     my @stats = mce_loop {
         my ($mce, $chunk_ref, $chunk_id) = @_;
         for (@{$chunk_ref}) {
-            my ($time, $res) = bench_run($benchmark);
+            my ($time, $res) = bench_run(@params);
             MCE->gather([$time, $res]);
         }
     }
@@ -447,13 +458,13 @@ sub mce_bench_run {
 }
 
 sub bench_run {
-    my ($benchmark, $srand) = @_;
+    my ($bench, $res, $srand) = @_;
     $srand //= 1;
     srand($srand); # For repeatability
     my $t0   = Time::HiRes::time();
-    my $out  = $benchmark->[2]->($benchmark->[3]);
+    my $out  = $bench->();
     my $time = sprintf("%.3f", Time::HiRes::time()-$t0);
-    my $r    = $out eq $benchmark->[0] ? 'Pass' : "Fail ($out)";
+    my $r    = !$res ? 'N/A' : $out eq $res->[0] ? 'Pass' : "Fail ($out)";
     return $time, $r;
 }
 
@@ -469,8 +480,8 @@ sub bench_astro {
 }
 
 sub bench_bioperl_codons {
-    my $skip = shift;
     my $iter = shift;
+    my $skip = shift;
     my $d    = Digest->new("MD5");
     my $file = catfile($datadir, "gbbct5.seq");
     foreach (1..$iter) {
@@ -617,6 +628,7 @@ sub bench_encode {
 
 sub bench_imager {
     my $iter = shift;
+    my $str  = _read_wiki_files();
     my $d    = Digest->new("MD5");
 
     my $data;
@@ -790,7 +802,7 @@ sub bench_moose {
     return md5_hex("$cnt objects");
 }
 
-sub bench_moose_prv {
+sub bench_moose_tests {
     my $iter = shift;
     my $tdir = catfile($datadir, 't');
     my $result;
@@ -807,11 +819,6 @@ sub bench_moose_prv {
     }
 }
 
-sub bench_primes_m {
-    my $iter = shift;
-    return bench_primes($iter, 7_500_000);
-}
-
 sub bench_primes {
     my $iter = shift;
     my $max  = shift;
@@ -820,22 +827,12 @@ sub bench_primes {
     return md5_hex(scalar(@primes)." primes up to $max");
 }
 
-sub bench_regex_asc {
-    my $iter = shift;
-    return bench_regex_subst($iter, '');
-}
-
-sub bench_regex_utf8 {
-    my $iter = shift;
-    return bench_regex_subst($iter, 'utf8');
-}
-
-sub bench_regex_subst {
+sub bench_regex_repl {
     my $iter  = shift;
-    my $enc   = shift;
+    my $enc   = shift || '';
     my $str   = _read_wiki_files($enc);
     my $match = bench_regex($str, $iter);
-    my $repl  = bench_subst($str, $iter);
+    my $repl  = bench_replace($str, $iter);
     return md5_hex($match, $repl);
 }
 
@@ -852,7 +849,7 @@ sub bench_regex {
     return "$count Matched";
 }
 
-sub bench_subst {
+sub bench_replace {
     my $str  = shift;
     my $iter = shift;
     my $count;
@@ -887,19 +884,19 @@ sub bench_timepiece {
 
 sub total_stats {
     my ($opt, $stats) = @_;
-    my $benchmarks = benchmark_list();
+    my ($benchmarks, $bench_results) = benchmark_list($opt->{quick});
     my $display = $opt->{time} ? 'times' : 'scores';
     my $title   = $opt->{time} ? 'Time (sec)' : 'Score';
     print "Aggregates:\n".pad_to("Benchmark",24).pad_to("Avg $title").pad_to("Min $title").pad_to("Max $title");
-    print pad_to("Pass %") unless $opt->{time};
+    print pad_to("Pass %") unless $opt->{quick};
     print "\n";
     foreach my $bench (sort keys %$benchmarks) {
         next unless $stats->{$bench}->{$display};
         my $str = calc_stats($opt, $stats->{$bench}->{$display});
         print pad_to("$bench:",24).$str;
         print pad_to(
-            sprintf("%d", 100 * ($opt->{iter}-($stats->{$bench}->{fail} || 0)) / $opt->{iter}))
-            unless $opt->{time};
+            sprintf("%d%%", 100 * ($opt->{iter}-($stats->{$bench}->{fail} || 0)) / $opt->{iter}))
+            unless $opt->{quick};
         print "\n";
     }
     my $str = calc_stats($opt, $stats->{total}->{$display});
@@ -1030,7 +1027,7 @@ sub rand_hash {
 
 # modified from https://github.com/famzah/langs-performance/blob/master/primes.pl
 sub _get_primes {
-    my $n = shift;
+    my $n = shift || 100_000_000;
     my @s = ();
     for (my $i = 3; $i < $n + 1; $i += 2) {
         push(@s, $i);
